package com.totopia.server.modules.user.controller;

import com.totopia.server.auth.payload.SignupRequest;
import com.totopia.server.commons.data.payload.ApiResponse;
import com.totopia.server.modules.user.entity.UserEntity;
import com.totopia.server.modules.user.payload.UserUpdateRequest;
import com.totopia.server.modules.user.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.web.SortDefault;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@RestController
public class UserController {

  @Autowired
  private UserService userService;

  @PostMapping(value = "/users")
  @ResponseStatus(code = HttpStatus.CREATED)
  public ResponseEntity<?> save(@Valid @RequestBody SignupRequest signupRequest) throws Exception{   // User user 파라미터는 추후에 수

    if (this.userService.existUserByUsername(signupRequest.getUsername())) {
      return new ResponseEntity<ApiResponse>(
          ApiResponse.builder().success(false).message("Username is already taken!").build(), HttpStatus.BAD_REQUEST);
    }

    this.userService.regist(signupRequest);

    return ResponseEntity.ok().body(new ApiResponse(true, "User registered successfully"));
  }

  @GetMapping(value = "/users")
  public Page<UserEntity> all(@RequestParam(value = "username", required = false) String username,
                              @SortDefault.SortDefaults({
                                  @SortDefault(
                                      sort = "createdAt",
                                      direction = Sort.Direction.DESC) })
                                  Pageable pageable) {

    return this.userService.getAllUsersByPageable(username, pageable);
  }

  @GetMapping(value = "/users/connect-users")
  public Page<UserEntity> connectUsers(@RequestParam(value = "username", required = false) String username,
                                       @SortDefault.SortDefaults({ @SortDefault(sort = "createdAt", direction = Sort.Direction.DESC) }) Pageable pageable) {
//    final List<String> curUsers = sessionRegistry.getAllPrincipals().stream()
//            .filter(u -> !sessionRegistry.getAllSessions(u, false).isEmpty())
//            .map(Object::toString)
//            .collect(Collectors.toList());
//
//    long start = pageable.getOffset();
//    long end = (start + pageable.getPageSize()) > curUsers.size() ? curUsers.size() : (start + pageable.getPageSize());
//    List<String> tempUsers = curUsers.subList((int)start, (int)end);
//    List<UserEntity> users = new ArrayList<>();
//
//    for (String conUser : tempUsers) {
//      UserEntity tempUser = this.userRepository.findByUsername(conUser).orElse(null);
//      if(tempUser != null) {
//        users.add(tempUser);
//      }
//    }
//    return new PageImpl<UserEntity>(users, new PageRequest(pageable.getPageNumber(), pageable.getPageSize()), users.size());
    return null;
  }
  @GetMapping(value = "/users/{userId}")
  public UserEntity findByUserId(@PathVariable Long userId) {
    return this.userService.getUserById(userId);
  }

  @DeleteMapping(value = "/users/{userId}")
  public ResponseEntity<?> deleteUser(@PathVariable Long userId) {

    this.userService.removeUserById(userId);
    return ResponseEntity.ok().build();
  }

  @PutMapping(value = "/users/{userId}")
  public ResponseEntity<?> updateUser(@PathVariable Long userId, @RequestBody UserUpdateRequest updateRequest) {

    UserEntity existrEntity = this.userService.getUserById(userId);

    if (existrEntity == null) {
      return new ResponseEntity<ApiResponse>(
          ApiResponse.builder().success(false).message("User ID does not exist").build(), HttpStatus.BAD_REQUEST);
    }

    UserEntity userEntity = this.userService.modifyByUser(existrEntity, updateRequest);

    return ResponseEntity.ok(userEntity);
//    return userRepository.findById(userId).map(user -> {
//
//      userRepository.save(user);
//      return ResponseEntity.ok(user);
//    }).orElseThrow(() -> new ResourceNotFoundException("User", "userId", userId));

  }


}
